@@ -1,21 +1,26 @@
 
+"""Application container - manages a list of available tasks to execute (all imported)
+"""
+
 import os
 import sys
 from datetime import datetime
-from typing import Dict, List, Union
+from typing import Dict, List, Union, Tuple
 from importlib.machinery import SourceFileLoader
 from traceback import print_exc
-from .syntax import TaskDeclaration
-from .syntax import TaskAliasDeclaration
-from .syntax import GroupDeclaration
-from .contract import ContextInterface
+from .api.syntax import TaskDeclaration
+from .api.syntax import TaskAliasDeclaration
+from .api.syntax import GroupDeclaration
+from .api.contract import ContextInterface
 from .argparsing import CommandlineParsingHelper
-from .inputoutput import SystemIO
+from .api.inputoutput import SystemIO
 from .exception import TaskNotFoundException
 from .exception import ContextFileNotFoundException
 from .exception import PythonContextFileNotFoundException
 from .exception import NotImportedClassException
-from .yaml_context import YamlParser
+from .exception import ContextException
+from .yaml_context import YamlSyntaxInterpreter
+from .yaml_parser import YamlFileLoader
 
 
 CURRENT_SCRIPT_PATH = os.path.dirname(os.path.realpath(__file__))
@@ -36,12 +41,16 @@ class ApplicationContext(ContextInterface):
     _task_aliases: Dict[str, TaskAliasDeclaration]
     _compiled: Dict[str, Union[TaskDeclaration, GroupDeclaration]]
     _created_at: datetime
+    _directory: str
+    directories: []
     io: SystemIO
 
-    def __init__(self, tasks: List[TaskDeclaration], aliases: List[TaskAliasDeclaration]):
+    def __init__(self, tasks: List[TaskDeclaration], aliases: List[TaskAliasDeclaration], directory: str):
         self._imported_tasks = {}
         self._task_aliases = {}
         self._created_at = datetime.now()
+        self._directory = directory
+        self.directories = [directory] if directory else []
 
         for task in tasks:
             self._add_component(task)
@@ -53,7 +62,7 @@ class ApplicationContext(ContextInterface):
     def merge(cls, first, second):
         """ Add one context to other context. Produces immutable new context. """
 
-        new_ctx = cls([], [])
+        new_ctx = cls([], [], '')
 
         for context in [first, second]:
             context: ApplicationContext
@@ -64,6 +73,8 @@ class ApplicationContext(ContextInterface):
             for name, task in context._task_aliases.items():
                 new_ctx._add_task(task)
 
+            new_ctx.directories += context.directories
+
         return new_ctx
 
     def compile(self) -> None:
@@ -134,7 +145,7 @@ class ContextFactory:
         if not os.path.isdir(path):
             raise Exception('Path "%s" not found' % path)
 
-        ctx = ApplicationContext([], [])
+        ctx = ApplicationContext([], [], path)
         contexts = []
 
         if os.path.isfile(path + '/makefile.py'):
@@ -158,8 +169,14 @@ class ContextFactory:
         makefile_path = path + '/' + filename
 
         with open(makefile_path, 'rb') as handle:
-            imports, tasks = YamlParser(self._io).parse(handle.read().decode('utf-8'), path, makefile_path)
-            return ApplicationContext(tasks=imports, aliases=tasks)
+            imported, tasks = YamlSyntaxInterpreter(self._io, YamlFileLoader([])).parse(
+                handle.read().decode('utf-8'), path, makefile_path
+            )
+
+            # Issue 33: Support mixed declarations in imports()
+            imports, aliases = distinct_imports(makefile_path, imported)
+
+            return ApplicationContext(tasks=imports, aliases=tasks + aliases, directory=path)
 
     @staticmethod
     def _load_from_py(path: str):
@@ -176,9 +193,13 @@ class ContextFactory:
             print_exc()
             raise NotImportedClassException(e)
 
+        # Issue 33: Support mixed declarations in imports()
+        imports, aliases = distinct_imports(makefile_path, makefile.IMPORTS if "IMPORTS" in dir(makefile) else [])
+
         return ApplicationContext(
-            tasks=makefile.IMPORTS if "IMPORTS" in dir(makefile) else [],
-            aliases=makefile.TASKS if "TASKS" in dir(makefile) else []
+            tasks=imports,
+            aliases=(makefile.TASKS if "TASKS" in dir(makefile) else []) + aliases,
+            directory=path
         )
 
     def create_unified_context(self, chdir: str = '') -> ApplicationContext:
@@ -207,7 +228,7 @@ class ContextFactory:
         # export for usage inside in makefiles
         os.environ['RKD_PATH'] = ":".join(paths)
 
-        ctx = ApplicationContext([], [])
+        ctx = ApplicationContext([], [], '')
 
         for path in paths:
             # not all paths could exist, we consider this, we look where it is possible
@@ -223,3 +244,27 @@ class ContextFactory:
         ctx.io = self._io
 
         return ctx
+
+
+def distinct_imports(file_path: str, imported: List[Union[TaskDeclaration, TaskAliasDeclaration]]) \
+        -> Tuple[List[TaskDeclaration], List[TaskAliasDeclaration]]:
+
+    """Separates TaskDeclaration and TaskAliasDeclaration into separate lists, doing validation by the way
+
+    :url: https://github.com/riotkit-org/riotkit-do/issues/33
+    """
+
+    aliases = []
+    imports = []
+
+    for declaration in imported:
+        if isinstance(declaration, TaskAliasDeclaration):
+            aliases.append(declaration)
+        elif isinstance(declaration, TaskDeclaration):
+            imports.append(declaration)
+        else:
+            raise ContextException('"imports()" from "%s" contains invalid type declaration "%s"' % (
+                file_path, str(declaration)
+            ))
+
+    return imports, aliases
