@@ -1,3 +1,4 @@
+from jsonschema import ValidationError
 
 
 class ContextException(Exception):
@@ -24,6 +25,12 @@ class EnvironmentVariableNotUsed(TaskException):
     pass
 
 
+class EnvironmentVariableNameNotAllowed(TaskException):
+    def __init__(self, var_name: str):
+        super().__init__('Environment variable with this name "' + var_name + '" cannot be declared, it probably a' +
+                         ' commonly reserved name by operating systems')
+
+
 class UserInputException(Exception):
     pass
 
@@ -36,6 +43,15 @@ class YamlParsingException(ContextException):
     """Logic or syntax errors in makefile.yaml"""
 
 
+class YAMLFileValidationError(YamlParsingException):
+    """Errors related to schema validation"""
+
+    def __init__(self, err: ValidationError):
+        super().__init__('YAML schema validation failed at path "%s" with error: %s' % (
+            '.'.join(err.path), err.message
+        ))
+
+
 class DeclarationException(ContextException):
     """Something wrong with the makefile.py/makefile.yaml """
 
